# -*- coding: utf-8 -*-

# Authors: Alejandro J. Cura <alecu@canonical.com>
# Authors: Natalia B Bidart <natalia.bidart@canonical.com>
#
# Copyright 2011 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

"""The user interface for the control panel for Ubuntu One."""

from __future__ import division

from PyQt4 import QtGui, QtCore
from twisted.internet import defer

from ubuntuone.controlpanel import backend
from ubuntuone.controlpanel.logger import setup_logging, log_call
from ubuntuone.controlpanel.gui import (
    humanize,
    EDIT_ACCOUNT_LINK,
    FACEBOOK_LINK,
    GET_SUPPORT_LINK,
    GREETING,
    PERCENTAGE_LABEL,
    TWITTER_LINK,
    USAGE_LABEL,
)
from ubuntuone.controlpanel.gui.qt import uri_hook
from ubuntuone.controlpanel.gui.qt.ui import controlpanel_ui


logger = setup_logging('qt.controlpanel')

NAME_STYLE = '<br><span style=" font-size:24pt;">%s!</span>'
PERCENTAGE_STYLE = '<span style=" font-size:16pt;">%.0f%%</span>'


class ControlPanel(QtGui.QWidget):
    """The Control Panel widget"""

    def __init__(self, parent=None):
        """Initialize the UI of the widget."""
        QtGui.QWidget.__init__(self, parent)
        self.ui = controlpanel_ui.Ui_Form()
        self.ui.setupUi(self)

        self.backend = backend.ControlBackend()
        self._setup()
        logger.debug('%s: started.', self.__class__.__name__)

    def _setup(self):
        """Do some extra setupping for the UI."""
        self.ui.get_more_space_button.uri = EDIT_ACCOUNT_LINK
        self.ui.help_button.uri = GET_SUPPORT_LINK

    # Invalid name "showEvent"
    # pylint: disable=C0103

    def showEvent(self, event):
        """Load info."""
        self.load()
        event.accept()

    @defer.inlineCallbacks
    def load(self):
        """Load info."""
        info = yield self.backend.account_info()
        self.process_info(info)

    @log_call(logger.debug)
    def process_info(self, info):
        """Process and display the account info."""
        name = NAME_STYLE % info['name']
        self.ui.greeting_label.setText(GREETING % {'user_display_name': name})

        used = int(info['quota_used'])
        total = int(info['quota_total'])
        percentage = {'percentage': PERCENTAGE_STYLE % ((used / total) * 100)}
        data = {'used': humanize(used), 'total': humanize(total)}
        self.ui.percentage_usage_label.setText(PERCENTAGE_LABEL % percentage)
        self.ui.quota_usage_label.setText(USAGE_LABEL % data)

    @QtCore.pyqtSlot()
    def on_twitter_button_clicked(self):
        """The twitter button was clicked."""
        uri_hook(TWITTER_LINK)

    @QtCore.pyqtSlot()
    def on_facebook_button_clicked(self):
        """The facebook button was clicked."""
        uri_hook(FACEBOOK_LINK)
