# -*- coding: utf-8 -*-

# Authors: Natalia B Bidart <natalia.bidart@canonical.com>
#
# Copyright 2011 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Tests for the Control Panel."""

from __future__ import division

from twisted.internet import defer

from ubuntuone.controlpanel.gui import qt
from ubuntuone.controlpanel.gui.qt import controlpanel as gui
from ubuntuone.controlpanel.gui.qt.tests import (
    BaseTestCase, SAMPLE_ACCOUNT_INFO, SAMPLE_NAME,
)


class ControlPanelTestCase(BaseTestCase):
    """Test the qt control panel."""

    innerclass_ui = gui.controlpanel_ui
    innerclass_name = "Ui_Form"
    class_ui = gui.ControlPanel

    @defer.inlineCallbacks
    def setUp(self):
        yield super(ControlPanelTestCase, self).setUp()
        self.ui.backend.next_result = SAMPLE_ACCOUNT_INFO

    def test_backend(self):
        """Backend is created."""
        self.assertIsInstance(self.ui.backend,
                              gui.backend.ControlBackend)

    @defer.inlineCallbacks
    def test_info_is_requested_on_load(self):
        """The info is requested to the backend."""
        yield self.ui.load()
        self.assert_backend_called('account_info')

    def test_process_info(self):
        """The info is processed when ready."""
        info = SAMPLE_ACCOUNT_INFO
        self.ui.process_info(info)

        name = gui.NAME_STYLE % SAMPLE_NAME
        greeting = gui.GREETING % {'user_display_name': name}
        self.assertEqual(self.ui.ui.greeting_label.text(), greeting)

        used = int(info['quota_used'])
        total = int(info['quota_total'])
        percentage = round((used / total) * 100, 2)
        percentage_usage = {'percentage': gui.PERCENTAGE_STYLE % percentage}
        expected = {'used': gui.humanize(used),
                    'total': gui.humanize(total)}
        msg = gui.USAGE_LABEL % expected
        self.assertEqual(self.ui.ui.quota_usage_label.text(), msg)
        msg = gui.PERCENTAGE_LABEL % percentage_usage
        self.assertEqual(self.ui.ui.percentage_usage_label.text(), msg)


class ExternalLinkButtonsTestCase(ControlPanelTestCase):
    """The link in the go-to-web buttons are correct."""

    @defer.inlineCallbacks
    def setUp(self):
        self.patch(qt, 'uri_hook', self._set_called)
        self.patch(gui, 'uri_hook', self._set_called)
        yield super(ExternalLinkButtonsTestCase, self).setUp()

    def test_get_more_space_button(self):
        """When clicking the get more GB button, the proper url is opened."""
        self.ui.ui.get_more_space_button.click()
        self.assertEqual(self._called, ((gui.EDIT_ACCOUNT_LINK,), {}))

    def test_help_button(self):
        """When clicking the help button, the proper url is opened."""
        self.ui.ui.help_button.click()
        self.assertEqual(self._called, ((gui.GET_SUPPORT_LINK,), {}))

    def test_twitter_button(self):
        """When clicking the twitter button, the proper url is opened."""
        self.ui.ui.twitter_button.click()
        self.assertEqual(self._called, ((gui.TWITTER_LINK,), {}))

    def test_facebook_button(self):
        """When clicking the facebook button, the proper url is opened."""
        self.ui.ui.facebook_button.click()
        self.assertEqual(self._called, ((gui.FACEBOOK_LINK,), {}))
